//<c>   Copyright 1987-2000 by Gerry J. Danen; all rights reserved.

/*
<t>     Display a visual tree of all directories on a drive

        With this utility you can view a visual tree of all the directories
        which you have access to.  It gives you a graphic overview of your
        system.
*/
#include"gd_tools.h"
#include"gd_extrn.h"
#include<direct.h>


#define pgVERS_ "V3.21"
#define pgTITL_ "Directory Tree"
#define pgCOPR_ "1987"


#if     (! LARGE_MODEL)
  #error LARGE MEMORY MODEL REQUIRED
#endif


//                                      graphics chars
#define         CH_179  ((UCHAR) '\xB3')        /*  */
#define         CH_191  ((UCHAR) '\xBF')        /*  */
#define         CH_192  ((UCHAR) '\xC0')        /*  */
#define         CH_195  ((UCHAR) '\xC3')        /*  */
#define         CH_196  ((UCHAR) '\xC4')        /*  */


//      The array of lines, called vline, is used for storing output that would
//      normally be printed on the screen.  The reason this is done is so that
//      FIX_TREE can manipulate the array and produce a nicer display.

#define LINES_M1        645
#define LINES_M2        100

UCHAR   temp[LINES_M2+1], vline[LINES_M1+1][LINES_M2+1];
int     i, count=0, deep=0;


void    find_dirs(void);
void    fix_tree(void);
void    print_tree(UCHAR drive);


//      FIND_DIRS searches for all directories.  Hidden ones and system ones
//      are also found.  It stores the information which it gathers into an
//      array of lines.  Most of the formatting takes place in this procedure,
//      but is later enhanced by FIX_TREE.


void    find_dirs( void )
{
    int         done ;
    struct find_t dta ;

    deep += 1 ;
    done = _dos_findfirst( "*.*", _A_SUBDIR+_A_HIDDEN+_A_SYSTEM, &dta );

    while ( ! done )
    {
        if ( ((dta.attrib & _A_SUBDIR) == _A_SUBDIR) && (dta.name[0] != '.') )
        {
            if ( count < LINES_M1 )
            {
                count += 1 ;
                if ( deep == 1 )
                    sprintf( temp, " %c", CH_195 );
                else
                    sprintf( temp, " %c%*s%c",
                                   CH_179, ((deep-1)*14)-1, " ", CH_195 );
                strcat( temp, dta.name );
                sprintf( vline[count], "%s", temp );
                _chdir( dta.name );
                find_dirs();
                deep -= 1 ;
                _chdir( ".." );
            }
            else
            {
                printf( "Maximum %d directories.  Extra ignored!\n", LINES_M1 );
            }
        }
        done = _dos_findnext(&dta);
    }
}


//      FIX_TREE is used for displaying the data which the previous procedure
//      formats.  It goes through the array of lines and figures out where
//      different graphics characters should be inserted to make the tree more
//      appealing.


void    fix_tree( void )
{
    int         j, k, llen, modified ;

    for ( i=1 ; i <= count ; i++ )              //  for each line...
    {
        llen = st_len( vline[i] );
        StrCatChars( vline[i], SPACE, LINES_M2, LINES_M2+1 );
        for ( j=0 ; j < llen ; j++ )            //  for each char in the line ...
        {
            modified = FALSE ;

            //  Insert Ŀ in line above if position above is blank

            if ( vline[i][j] == CH_195 )
            {
                if ( vline[i-1][j] == SPACE )   //  char on line above = space?
                {
                    strcpy( temp, vline[i-1] );
                    st_rtrim( temp );
                    for ( k = strlen(temp); k < j; k++ )
                    {
                        strcat( temp, "\xC4" );         //  196
                        modified = TRUE ;
                    }
                    if ( modified )
                        strcat( temp, "\xBF" );         //  191
                    sprintf( vline[i-1], "%s", temp );
                }
            }
            if ( (vline[i][j] == CH_195 ) && (vline[i+1][j] == SPACE ) )
            {
                strcpy( temp, vline[i] );
                st_rtrim( temp );
                for ( k = strlen(temp) ; k >=0 ; k-- )
                {
                    if ( temp[k] == CH_195 )
                        temp[k] = CH_192 ;
                }
                sprintf( vline[i], "%s", temp );
            }

            //  Connect vertical line upward (backward in array)

            if ( ((vline[i][j] == CH_195 ) || (vline[i][j] == CH_192 ))
                 && (vline[i-1][j] == SPACE) )
            {
                k = i - 1;
                while ( vline[k][j] != CH_192 )
                {
                    vline[k][j] = CH_179 ;
                    k--;
                }
                vline[k][j] = CH_195 ;
            }
        }
    }

    //  Clean up bottom part of root-directory line

    i = count ;
    if ( vline[i][1] != CH_192 )
    {
        while ( (i >= 0) && (vline[i][1] != CH_195 ) )
        {
            if ( vline[i][1] == CH_179 )
                vline[i][1] = ' ' ;
            i -= 1 ;
        }
        vline[i][1] = CH_192 ;
    }
}


void    print_tree( UCHAR drv )
{
    UCHAR       vl_drive[3], vl_label[13], str1[21], str2[21] ;
    INT32       vl_date, vl_time ;
    int         x ;

    sprintf( str1, "%c:\\", drv );
    printf ( "Drive %c:  ", drv );
    if ( ds_vol_label( str1, vl_drive, vl_label, &vl_date, &vl_time ) == OK )
    {
        FormatDate( vl_date, str2, 1 );
        FormatTime( vl_time, str1, 0, 0 );
        printf( "Volume label \"%s\" %s %s\n", vl_label, str2, str1 );
    }
    else
    {
        printf( "(no volume label)\n" );
    }

    for ( x=0 ; x <= count ; x++ )
    {
        st_rtrim( vline[x] );
        printf( "%s\n", vline[x] );
    }
}


//      The main section is used for initialization and for calling the most
//      important part of this utility, FIND_DIRS.  Once FIND_DIRS has done its
//      work and has created a preliminary visual tree, the FIX_TREE procedure
//      is called.  It fixes up any minor details that are necessary and then
//      PRINT_TREE displays the visual tree on the screen.


void    main( int argc, char *argv[] )
{
    UCHAR       drive, target_drive, origdir[_MAX_PATH], orig1dir[_MAX_PATH];
    unsigned    numofdrives ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);

    _getcwd( origdir, _MAX_PATH );              // get current directory
    strcpy( orig1dir, origdir );                // make copy
    if ( argc > 1 )
    {
        target_drive = (UCHAR) toupper( argv[1][0] );
        if ( target_drive < 'A' || target_drive > 'Z' )
        {
            fprintf( stderr, "\n\t\tUsage:  %s [drive]\n", get_global_program_name() );
            exit( 1 );
        }
        _dos_setdrive( target_drive - 64, &numofdrives );
        _getcwd( orig1dir, _MAX_PATH );         //  save current directory
    }
    else
    {
        strcpy( orig1dir, origdir );            //  make copy of directory name
    }

    fprintf( stderr, "Wait..." );
//    memset( vline, ((int) ' '), (size_t) ((LINES_M1+1)*(LINES_M2+1)) );
    for ( i=0 ; i < LINES_M1 ; i++ )
    {
        memset( vline[i], ((int) ' '), (size_t) LINES_M2 );
        vline[i][LINES_M2] = EOS ;
    }

    drive = orig1dir[0] ;
    _chdir( "\\" );                             //  switch to root
    sprintf( vline[count], "\\%c", CH_191 );    //  a back slash and ASCII 191
    find_dirs();
    _chdir( orig1dir );                         //  back to original on target
    _dos_setdrive( origdir[0] - 64, &numofdrives );
    _chdir( origdir );                          //  back to original
    fprintf( stderr, "..." );
    fix_tree();
    fprintf( stderr, "\r\t\t\r" );
    print_tree( drive );
    exit( 0 );
}
